@@ -30,15 +30,20 @@
 # and partially modified. The code in question is licensed under MIT license.
 
 import logging
+import os
 import pilkit.processors
 import sys
 
+from copy import deepcopy
+from datetime import datetime
+from PIL.ExifTags import TAGS, GPSTAGS
 from PIL import Image as PILImage
 from PIL import ImageDraw, ImageOps
-from PIL.ExifTags import TAGS, GPSTAGS
 from pilkit.processors import Transpose, Adjust
 from pilkit.utils import save_image
-from datetime import datetime
+
+from . import compat
+from .settings import get_thumb
 
 
 def _has_exif_tags(img):
@@ -61,13 +66,16 @@ def generate_image(source, outname, settings, options=None):
 
     # Preserve EXIF data
     if settings['copy_exif_data'] and _has_exif_tags(img):
-        options = options or {}
+        if options is not None:
+            options = deepcopy(options)
+        else:
+            options = {}
         options['exif'] = img.info['exif']
 
     # Rotate the img, and catch IOError when PIL fails to read EXIF
     try:
         img = Transpose().process(img)
-    except IOError:
+    except (IOError, IndexError):
         pass
 
     # Resize the image
@@ -91,8 +99,7 @@ def generate_image(source, outname, settings, options=None):
 
     outformat = img.format or original_format or 'JPEG'
     logger.debug(u'Save resized image to {0} ({1})'.format(outname, outformat))
-    with open(outname, 'w') as fp:
-        save_image(img, fp, outformat, options=options, autoconvert=True)
+    save_image(img, outname, outformat, options=options, autoconvert=True)
 
 
 def generate_thumbnail(source, outname, box, fit=True, options=None):
@@ -109,8 +116,29 @@ def generate_thumbnail(source, outname, box, fit=True, options=None):
 
     outformat = img.format or original_format or 'JPEG'
     logger.debug(u'Save thumnail image: {0} ({1})'.format(outname, outformat))
-    with open(outname, 'w') as fp:
-        save_image(img, fp, outformat, options=options, autoconvert=True)
+    save_image(img, outname, outformat, options=options, autoconvert=True)
+
+
+def process_image(filepath, outpath, settings):
+    """Process one image: resize, create thumbnail."""
+
+    filename = os.path.split(filepath)[1]
+    outname = os.path.join(outpath, filename)
+    ext = os.path.splitext(filename)[1]
+
+    if ext in ('.jpg', '.jpeg', '.JPG', '.JPEG'):
+        options = settings['jpg_options']
+    elif ext == '.png':
+        options = {'optimize': True}
+    else:
+        options = {}
+
+    generate_image(filepath, outname, settings, options=options)
+
+    if settings['make_thumbs']:
+        thumb_name = os.path.join(outpath, get_thumb(settings, filename))
+        generate_thumbnail(outname, thumb_name, settings['thumb_size'],
+                           fit=settings['thumb_fit'], options=options)
 
 
 def add_copyright(img, text):
@@ -154,12 +182,12 @@ def get_exif_tags(source):
 
     logger = logging.getLogger(__name__)
 
-    if not '.jpg' in source.lower():
+    if os.path.splitext(source)[1].lower() not in ('.jpg', '.jpeg'):
         return (None, None)
 
     try:
         data = _get_exif_data(source)
-    except (TypeError, IOError):
+    except (IOError, IndexError, TypeError, AttributeError):
         logger.warning(u'Could not read EXIF data from {0}'.format(source))
         return (None, None)
 
@@ -185,7 +213,12 @@ def get_exif_tags(source):
             # Remove null bytes at the end if necessary
             date = data['DateTimeOriginal'].rsplit('\x00')[0]
             dt = datetime.strptime(date, '%Y:%m:%d %H:%M:%S')
-            simple['datetime'] = dt
+            dt = dt.strftime('%A, %d. %B %Y')
+
+            if compat.PY2:
+                simple['datetime'] = dt.decode('utf8')
+            else:
+                simple['datetime'] = dt
         except (ValueError, TypeError) as e:
             msg = u'Could not parse DateTimeOriginal of %s: %s' % (source, e)
             logger.warning(msg)
